package it.polimi.deib.se.swing.pizza.model;

import it.polimi.deib.se.swing.pizza.model.ui.PizzaSelectionUI;

import java.util.HashSet;
import java.util.Set;

/** Un modello di selezione di un ordine per una pizza. */
public class PizzaSelectionModel {
  // stato della selezione dell'ordine.
  private PizzaSize size = null;
  private PizzaFlour flour = null;
  private Set<Ingredient> ingredients;
  // interfaccia utente
  private PizzaSelectionUI ui;

  public PizzaSelectionModel() {
    this.ingredients = new HashSet<>();
  }

  public void setUI(PizzaSelectionUI ui) {
    this.ui = ui;
    this.updateUI();
  }

  public boolean sizeSelected() {
    return this.size != null;
  }

  public boolean flourSelected() {
    return this.flour != null;
  }

  public boolean isAValidSelection() {
    return this.sizeSelected() &&
        this.flourSelected();
  }

  public void setSize(PizzaSize size) {
    this.size = size;
    updateUI();
  }

  public void setFlour(PizzaFlour flour) {
    this.flour = flour;
    updateUI();
  }

  public void addIngredient(Ingredient ingredient) {
    if (ingredient == null)
      throw new IllegalArgumentException("ingredient cannot be null");
    this.ingredients.add(ingredient);
    updateUI();
  }

  public void removeIngredient(Ingredient ingredient) {
    this.ingredients.remove(ingredient);
    updateUI();
  }

  private void updateUI() {
    if (this.ui != null) {
      this.ui.setValid(this.isAValidSelection());
      if (this.isAValidSelection())
        this.ui.setCaloriesForSelection(this.getCalories());
    }
  }

  public double getCalories() {
    if (!this.isAValidSelection())
      throw new IllegalStateException();

    double calories = 0;

    calories = this.size.weight() * (this.flour.caloriesPer100Grams() / 100.0);

    for (Ingredient ingredient: this.ingredients)
      calories += ingredient.getCalories();

    return calories;
  }

  public void reset() {
    this.size = null;
    this.flour = null;
    this.ingredients.clear();
    if (ui != null)
      ui.reset();
    updateUI();		
  }
}
