package it.polimi.deib.se.swing.pizza.model.ui.gui;

import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.util.ArrayList;
import java.util.List;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;

import it.polimi.deib.se.swing.pizza.model.Ingredient;
import it.polimi.deib.se.swing.pizza.model.PizzaFlour;
import it.polimi.deib.se.swing.pizza.model.PizzaSelectionModel;
import it.polimi.deib.se.swing.pizza.model.PizzaSize;
import it.polimi.deib.se.swing.pizza.model.ui.PizzaSelectionUI;
import it.polimi.deib.se.swing.pizza.model.ui.controller.FlourSelectionHandler;
import it.polimi.deib.se.swing.pizza.model.ui.controller.IngredientsSelectionHandler;
import it.polimi.deib.se.swing.pizza.model.ui.controller.OrderButtonHandler;
import it.polimi.deib.se.swing.pizza.model.ui.controller.SizeSelectionHandler;

public class PizzaSelectionGUI implements PizzaSelectionUI {

  private final JFrame jframe;
  private final PizzaSelectionModel model;

  private final List<JCheckBox> ingredientsCheckBoxes;
  private final List<JRadioButton> sizeButtons;
  private final List<JRadioButton> flourButtons;

  private ButtonGroup sizeButtonsGroup, flourButtonsGroup;

  private JLabel caloriesLabel;
  private JButton orderButton;
  private JPanel ingredientsPanel, flourPanel, sizePanel;

  public PizzaSelectionGUI(PizzaSelectionModel model) {
    this.model = model;

    this.jframe = new JFrame("Pizza order selection");

    this.ingredientsCheckBoxes = new ArrayList<>();
    this.sizeButtons = new ArrayList<>();
    this.flourButtons = new ArrayList<>();

    initPanels();
    initSizeButtons();
    initFlourButtons();
    initIngredientsButtons();

    jframe.pack();
    /* centra il frame nello schermo */
    jframe.setLocationRelativeTo(null);
    jframe.setVisible(true);
  }
  
  // ==============================================================
  // METODI PRIVATI PER LA COSTRUZIONE DELLA GUI
  // ==============================================================

  private void initPanels() {
    /* GridLayout principale */
    this.jframe.getContentPane().setLayout(new GridLayout(5,1));

    /* prima riga: label calorie */
    this.caloriesLabel = new JLabel();
    this.jframe.add(this.caloriesLabel);

    /* seconda riga: panel dimensione */
    this.sizePanel = new JPanel();
    this.jframe.add(this.sizePanel);

    /* terza riga: panel farina */
    this.flourPanel = new JPanel();
    this.jframe.add(this.flourPanel);

    /* quarta riga: panel ingredienti */
    this.ingredientsPanel = new JPanel();
    this.jframe.add(this.ingredientsPanel);

    /* sesta riga: pulsante ordine */
    this.orderButton = new JButton("Place Order");
    this.orderButton.addActionListener(new OrderButtonHandler(this.jframe, model));
    this.jframe.add(this.orderButton);
  }

  private void initSizeButtons() {
    this.sizePanel.setLayout(new FlowLayout());
    this.sizeButtonsGroup = new ButtonGroup();

    for (PizzaSize size: PizzaSize.values()) {
      JRadioButton sizeButton = new JRadioButton(size.toString());
      this.sizeButtons.add(sizeButton);
      this.sizePanel.add(sizeButton);
      this.sizeButtonsGroup.add(sizeButton);
      sizeButton.addItemListener(new SizeSelectionHandler(model, size));
    }
  }

  private void initFlourButtons() {
    this.flourPanel.setLayout(new FlowLayout());
    this.flourButtonsGroup = new ButtonGroup();

    for (PizzaFlour flour: PizzaFlour.values()) {
      JRadioButton flourButton = new JRadioButton(flour.toString());
      this.flourButtons.add(flourButton);
      this.flourPanel.add(flourButton);
      this.flourButtonsGroup.add(flourButton);
      
      // Per JCheckBox e JRadioButton, meglio usare un ItemListener:
      // http://docs.oracle.com/javase/6/docs/api/java/awt/event/ItemListener.html
      flourButton.addItemListener(new FlourSelectionHandler(model, flour));
    }
  }

  private void initIngredientsButtons() {
    int nIngredients = Ingredient.values().length;
    this.ingredientsPanel.setLayout(new GridLayout(nIngredients/3 + 1,3));

    for (Ingredient ingredient: Ingredient.values()) {
      JCheckBox ingredientButton = new JCheckBox(ingredient.toString());
      this.ingredientsPanel.add(ingredientButton);
      this.ingredientsCheckBoxes.add(ingredientButton);
      ingredientButton.addItemListener(new IngredientsSelectionHandler(model, ingredient));
    }
  }

  // ==============================================================
  // METODI CHIAMATI DAL MODELLO PER L'AGGIORNAMENTO DELLA VISTA
  // ==============================================================
  
  @Override
  public void setValid(boolean valid) {
    if (valid) {
      this.orderButton.setEnabled(true);
    } else {
      this.orderButton.setEnabled(false);
    }
  }

  @Override
  public void setCaloriesForSelection(double calories) {
    this.caloriesLabel.setText("Calories: " + String.valueOf(calories));
  }

  @Override
  public void reset() {
    this.sizeButtonsGroup.clearSelection();
    this.flourButtonsGroup.clearSelection();
    for (JCheckBox ingredientBox: this.ingredientsCheckBoxes)
      ingredientBox.setSelected(false);
    this.caloriesLabel.setText("");
  }

}
